@@ -1,19 +1,13 @@
 #!/usr/bin/perl
-# $Id: bench.pl,v 1.24 2007/10/27 10:23:34 tinita Exp $
+# $Id: bench.pl 1126 2011-10-31 19:56:35Z tinita $
 use strict;
 use warnings;
-use lib qw(blib/lib ../blib/lib);
 use Getopt::Long;
+use Benchmark qw/ timethese cmpthese /;
+use FindBin qw/ $RealBin /;
+chdir "$RealBin/..";
 #use Devel::Size qw(size total_size);
 my $count = 0;
-my $ht_file = 'test.htc';
-#$ht_file = 'test.htc.10';
-#$ht_file = 'test.htc.20';
-my $htcc_file = $ht_file . 'c';
-my $tt_file = "test.tt";
-#$tt_file = "examples/test.tt.10";
-#$tt_file = "examples/test.tt.20";
-my $tst_file = "examples/test.tst";
 mkdir "cache";
 mkdir "cache/htc";
 mkdir "cache/htcc";
@@ -22,16 +16,20 @@ mkdir "cache/htpl";
 mkdir "cache/jit";
 my %use = (
 	'HTML::Template'           => 0,
-	'HTML::Template::Pro'      => 0,
+    'HTML::Template::Pro'      => 0,
 	'HTML::Template::Compiled' => 0,
 	'HTML::Template::Pluggable' => 0,
 	'HTML::Template::Expr' => 0,
 	'HTML::Template::Compiled::Classic' => 0,
 #	'HTML::Template::JIT'      => 0,
 	'Template'                 => 0,
+    'Template::HTML'           => 0,
+    'Template::AutoFilter'     => 0,
+    'Template::Like'           => 0,
 	'CGI::Ex::Template'        => 0,
 	# not yet
 	'Text::ScriptTemplate'     => 0,
+    'Text::Xslate'             => 0,
 );
 for my $key (sort keys %use) {
 	eval "require $key";
@@ -39,9 +37,8 @@ for my $key (sort keys %use) {
 	my $version = $use{$key} ? $key->VERSION : "-";
     printf "using %35s %s\n", $key, $version;
 }
-HTML::Template::Compiled->ExpireTime(100);
 HTML::Template::Compiled->clear_filecache("cache/htc");
-use Benchmark;
+HTML::Template::Compiled->clear_filecache("cache/htcc");
 my $debug = 0;
 $ENV{'HTML_TEMPLATE_ROOT'} = "examples";
 my $FILE_CACHE     = 0;
@@ -50,6 +47,8 @@ my $LOOP_CONTEXT   = 1;
 my $GLOBAL_VARS    = 0;
 my $CASE_SENSITIVE = 1;
 my $default_escape = 0;
+my $template_size  = 1;
+my $bench          = 'timethese';
 GetOptions(
     "file-cache=i" => \$FILE_CACHE,
     "mem-cache=i" => \$MEM_CACHE,
@@ -57,17 +56,33 @@ GetOptions(
     "global-vars=i" => \$GLOBAL_VARS,
     "case-sensitive=i" => \$CASE_SENSITIVE,
     "default-escape=i" => \$default_escape,
+    "template-size=i" => \$template_size,
+    "bench=s" => \$bench,
 );
 my $iterations = shift;
+my $ht_file = 'test.htc';
+my $htcc_file = $ht_file . 'c';
+my $tt_file = "test.tt";
+my $tst_file = "test.tst";
+$template_size =~ tr/0-9//cd;
+if ($template_size > 1) {
+    for my $file ($ht_file, $htcc_file, $tt_file, $tst_file) {
+        open my $fh, "<", "examples/$file" or die "examples/$file: $!";
+        my $data = do { local $/; <$fh> };
+        my $new_file = "$file.n$template_size";
+        open my $out, ">", "examples/$new_file" or die $!;
+        print $out $data x $template_size;
+        $file = $new_file;
+    }
+}
 
 print "running with:
-file-cache:     $FILE_CACHE
-mem-cache:      $MEM_CACHE
-loop-context:   $LOOP_CONTEXT
-global-vars:    $GLOBAL_VARS
-case-sensitive: $CASE_SENSITIVE
+--file-cache $FILE_CACHE --mem-cache $MEM_CACHE --loop-context $LOOP_CONTEXT "
+. "--global-vars $GLOBAL_VARS --case-sensitive $CASE_SENSITIVE "
+. "--default-escape $default_escape --template-size=$template_size
 ";
 
+my $STDOUT = 0;
 sub new_htc {
 	my $t1 = HTML::Template::Compiled->new_file( $ht_file,
 		#path => 'examples',
@@ -77,11 +92,15 @@ sub new_htc {
 		debug => $debug,
 		# note that you have to create the cachedir
 		# first, otherwise it will run without cache
-        cache_dir => ($FILE_CACHE ? "cache/htc" : undef),
+#        cache_dir => ($FILE_CACHE ? "cache/htc" : undef),
+        file_cache_dir => ($FILE_CACHE ? "cache/htc" : undef),
+        file_cache => ($FILE_CACHE ? 1 : undef),
+
         cache => $MEM_CACHE,
-		out_fh => 1,
+		out_fh => $STDOUT ? 1 : 0,
         global_vars => $GLOBAL_VARS,
         tagstyle => [qw(-asp -comment)],
+		expire_time => 1000,
 	);
 	return $t1;
 }
@@ -97,10 +116,11 @@ sub new_htcc {
 		# first, otherwise it will run without cache
         cache_dir => ($FILE_CACHE ? "cache/htcc" : undef),
         cache => $MEM_CACHE,
-		out_fh => 1,
+		out_fh => $STDOUT ? 1 : 0,
         global_vars => $GLOBAL_VARS,
         debug => 0,
         tagstyle => [qw(-asp -comment)],
+		expire_time => 1000,
         #debug => 1,
 	);
 	return $t1;
@@ -108,7 +128,7 @@ sub new_htcc {
 
 sub new_tst {
 	my $t = Text::ScriptTemplate->new();
-	$t->load($tst_file);
+	$t->load("examples/$tst_file");
 	#my $size = total_size($t1);
 	#print "size htc = $size\n";
 	return $t;
@@ -144,6 +164,7 @@ sub new_ht {
         file_cache_dir => 'cache/ht') : (),
         global_vars => $GLOBAL_VARS,
         die_on_bad_params => 0,
+		blind_cache => 1,
 	);
 	return $t2;
 }
@@ -193,6 +214,26 @@ sub new_htj {
 	return $t2;
 }
 
+sub new_tl {
+	my $tt= Template::Like->new(
+    );
+
+#        $FILE_CACHE
+#            ? (
+#                COMPILE_EXT => '.ttc',
+#                COMPILE_DIR => 'cache/tt',
+#            )
+#            : (),
+#        $MEM_CACHE
+#            ? ()
+#            : (CACHE_SIZE => 0),
+#		INCLUDE_PATH => 'examples',
+
+	#my $size = total_size($tt);
+	#print "size tt  = $size\n";
+	return $tt;
+}
+
 sub new_tt {
 	my $tt= Template->new(
         $FILE_CACHE
@@ -211,6 +252,54 @@ sub new_tt {
 	return $tt;
 }
 
+sub new_ttaf {
+	my $tt= Template::AutoFilter->new(
+        $FILE_CACHE
+            ? (
+                COMPILE_EXT => '.ttc',
+                COMPILE_DIR => 'cache/tt',
+            )
+            : (),
+        $MEM_CACHE
+            ? ()
+            : (CACHE_SIZE => 0),
+		INCLUDE_PATH => 'examples',
+	);
+	#my $size = total_size($tt);
+	#print "size tt  = $size\n";
+	return $tt;
+}
+
+sub new_tth {
+	my $tt= Template::HTML->new(
+        $FILE_CACHE
+            ? (
+                COMPILE_EXT => '.ttc',
+                COMPILE_DIR => 'cache/tt',
+            )
+            : (),
+        $MEM_CACHE
+            ? ()
+            : (CACHE_SIZE => 0),
+		INCLUDE_PATH => 'examples',
+	);
+	#my $size = total_size($tt);
+	#print "size tt  = $size\n";
+	return $tt;
+}
+
+sub new_xslate {
+	my $t = Text::Xslate->new(
+        cache_dir => ($FILE_CACHE ? "cache/xslate" : undef),
+        cache => $MEM_CACHE,
+		path => 'examples',
+        syntax => 'TTerse',
+	);
+	#my $size = total_size($tt);
+	#print "size tt  = $size\n";
+	return $t;
+}
+
 sub new_cet {
 	my $tt= CGI::Ex::Template->new(
         $FILE_CACHE
@@ -257,39 +346,106 @@ open OUT, ">>/dev/null";
 sub output {
 	my $t = shift;
 	return unless defined $t;
-	$params{name} = (ref $t).' '.$count++;
 	$t->param(%params);
-	#print $t->{code} if exists $t->{code};
-    my $out = $t=~m/Compiled/?$t->output(\*OUT):$t->output;
-    #my $out = $t=~m/Compiled/?$t->output(\*STDOUT):$t->output;
-	print OUT $out;
+    if ($STDOUT) {
+        my $out;
+        if ($t=~m/Compiled/) {
+            $out = $t->output(\*OUT);
+        }
+        else {
+            $out = $t->output;
+        }
+        print OUT $out;
+    }
+    else {
+        my $out = $t->output();
+    }
 	#print "output():$out\n";
 	#my $size = total_size($t);
 	#print "size $t = $size\n";
-	#print "\nOUT: $out";
 }
 
-#open TT_OUT, ">&STDOUT";
 sub output_tst {
 	my $t = shift;
 	return unless defined $t;
-	#warn Data::Dumper->Dump([\%params], ['params']);
 	$t->setq(%params,tmpl=>$t);
 	my $out = $t->fill;
 	#print "output_tst():$out\n";
-	print OUT $out;
+    if ($STDOUT) {
+        print OUT $out;
+    }
 }
+sub output_tl {
+	my $t = shift;
+	return unless defined $t;
+    chdir 'examples';
+	my $filett = $tt_file;
+    if ($STDOUT) {
+        $t->process($filett, \%params, \*OUT) or die $t->error();
+    }
+    else {
+        my $out;
+        $t->process($filett, \%params, \$out) or die $t->error();
+    }
+	#my $size = total_size($t);
+	#print "size $t = $size\n";
+	#my $out = $t->output;
+	#print "\nOUT: $out";
+    chdir '..';
+}
+
 sub output_tt {
 	my $t = shift;
 	return unless defined $t;
 	my $filett = $tt_file;
 	#$t->process($filett, \%params, \*OUT);
-	$t->process($filett, \%params, \*OUT) or die $t->error();
+    if ($STDOUT) {
+        $t->process($filett, \%params, \*OUT) or die $t->error();
+    }
+    else {
+        my $out;
+        $t->process($filett, \%params, \$out) or die $t->error();
+    }
 	#my $size = total_size($t);
 	#print "size $t = $size\n";
 	#print $t->{code} if exists $t->{code};
-	#my $out = $t->output;
-	#print "\nOUT: $out";
+    #my $out = $t->output;
+    #print "\nOUT: $out";
+}
+
+sub output_ttaf {
+	my $t = shift;
+	return unless defined $t;
+	my $filett = $tt_file;
+	#$t->process($filett, \%params, \*OUT);
+    if ($STDOUT) {
+        $t->process($filett, \%params, \*OUT) or die $t->error();
+    }
+    else {
+        my $out;
+        $t->process($filett, \%params, \$out) or die $t->error();
+    }
+	#my $size = total_size($t);
+	#print "size $t = $size\n";
+	#print $t->{code} if exists $t->{code};
+    #my $out = $t->output;
+    #print "\nOUT: $out";
+}
+
+sub output_xslate {
+	my $t = shift;
+	return unless defined $t;
+	my $filett = $tt_file;
+	#$t->process($filett, \%params, \*OUT);
+#	$t->process($filett, \%params, \*OUT) or die $t->error();
+	#my $size = total_size($t);
+	#print "size $t = $size\n";
+	#print $t->{code} if exists $t->{code};
+    my $out = $t->render('test.xslate', \%params);
+    #my $out = $t->output;
+    if ($STDOUT) {
+        print OUT $out;
+    }
 }
 
 my $global_htc = $use{'HTML::Template::Compiled'} ? new_htc : undef;
@@ -299,12 +455,15 @@ my $global_htp = $use{'HTML::Template::Pro'} ? new_htp : undef;
 my $global_htpl = $use{'HTML::Template::Pluggable'} ? new_htpl : undef;
 my $global_htj = $use{'HTML::Template::JIT'} ? new_htj : undef;
 my $global_tt = $use{'Template'} ? new_tt : undef;
+my $global_ttaf = $use{'Template::AutoFilter'} ? new_ttaf : undef;
+my $global_tth = $use{'Template::HTML'} ? new_tth : undef;
+my $global_xslate = $use{'Text::Xslate'} ? new_xslate : undef;
+my $global_tl = $use{'Template::Like'} ? new_tl : undef;
 my $global_cet = $use{'CGI::Ex::Template'} ? new_cet : undef;
 my $global_tst = $use{'Text::ScriptTemplate'} ? new_tst : undef;
 if(1) {
-    #Benchmark::cmpthese ($iterations||-1, {
-    timethese ($iterations||-1, {
-        $use{'HTML::Template::Compiled'} ? (
+my %args = (
+       $use{'HTML::Template::Compiled'} ? (
             # deactivate memory cache
             #new_htc_w_clear_cache => sub {my $t = new_htc();$t->clear_cache},
             # normal, with memory cache
@@ -353,6 +512,33 @@ if(1) {
                 ? ()
                 : (all_tt_new_object => sub {my $t = new_tt();output_tt($t)}),
         ): (),
+        $use{'Template::AutoFilter'} ? (
+            #new_ttaf => sub {my $t = new_ttaf();},
+            #output_ttaf => sub {output_ttaf($global_ttaf)},
+            process_ttaf => sub {output_ttaf($global_ttaf)},
+            $MEM_CACHE
+                ? ()
+                : (all_ttaf_new_object => sub {my $t = new_ttaf();output_ttaf($t)}),
+        ): (),
+        $use{'Template::HTML'} ? (
+            #new_tt => sub {my $t = new_tt();},
+            #output_tt => sub {output_tt($global_tt)},
+            process_tth => sub {output_tt($global_tth)},
+            $MEM_CACHE
+                ? ()
+                : (all_tth_new_object => sub {my $t = new_tth();output_tt($t)}),
+        ): (),
+        $use{'Text::Xslate'} ? (
+            #new_tt => sub {my $t = new_tt();},
+            #output_tt => sub {output_tt($global_tt)},
+            process_xslate => sub {output_xslate($global_xslate)},
+            $MEM_CACHE
+                ? ()
+                : (all_xslate_new_object => sub {my $t = new_xslate();output_xslate($t)}),
+        ): (),
+        $use{'Template::Like'} ? (
+            process_tl => sub {output_tl($global_tl)},
+        ): (),
         $use{'CGI::Ex::Template'} ? (
             #new_tt => sub {my $t = new_tt();},
             #output_tt => sub {output_tt($global_tt)},
@@ -366,6 +552,16 @@ if(1) {
                     #output_tst => sub {output_tst($global_tst)},
 						all_tst => sub {my $t = new_tst();output_tst($t)},
         ): (),
-	});
+);
+if ($bench eq 'timethese') {
+    timethese ($iterations||-1, {
+        %args
+ 	});
+}
+elsif ($bench eq 'cmpthese') {
+    cmpthese ($iterations||-1, {
+        %args
+ 	});
+}
 }
 __END__
